@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    jpgimage.hpp
   @brief   Class JpegImage to access JPEG images
-  @version $Rev: 2045 $
+  @version $Rev: 2558 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @author  Brad Schick (brad)
@@ -66,6 +66,7 @@ namespace Exiv2 {
         static const char     ps3Id_[]; //!< %Photoshop marker
         static const char     bimId_[]; //!< %Photoshop marker
         static const uint16_t iptc_;    //!< %Photoshop IPTC marker
+        static const uint16_t preview_; //!< %Photoshop preview marker
 
         /*!
           @brief Validates all IRBs
@@ -108,6 +109,14 @@ namespace Exiv2 {
                                  const byte **record,
                                  uint32_t *const sizeHdr,
                                  uint32_t *const sizeData);
+        /*!
+          @brief Forwards to locatePreviewIrb() with \em psTag = \em preview_
+         */
+        static int locatePreviewIrb(const byte *pPsData,
+                                    long sizePsData,
+                                    const byte **record,
+                                    uint32_t *const sizeHdr,
+                                    uint32_t *const sizeData);
         /*!
           @brief Set the new IPTC IRB, keeps existing IRBs but removes the
                  IPTC block if there is no new IPTC data to write.
