@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    value.hpp
   @brief   Value interface and concrete subclasses
-  @version $Rev: 2073 $
+  @version $Rev: 2453 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created
@@ -40,6 +40,7 @@
 #include <vector>
 #include <map>
 #include <iostream>
+#include <iomanip>
 #include <sstream>
 #include <memory>
 #include <cstring>
@@ -217,8 +218,8 @@ namespace Exiv2 {
           <TR><TD class="indexkey">signedShort</TD><TD class="indexvalue">%ValueType &lt; int16_t &gt;</TD></TR>
           <TR><TD class="indexkey">signedLong</TD><TD class="indexvalue">%ValueType &lt; int32_t &gt;</TD></TR>
           <TR><TD class="indexkey">signedRational</TD><TD class="indexvalue">%ValueType &lt; Rational &gt;</TD></TR>
-          <TR><TD class="indexkey">tiffFloat</TD><TD class="indexvalue">%DataValue(tiffFloat)</TD></TR>
-          <TR><TD class="indexkey">tiffDouble</TD><TD class="indexvalue">%DataValue(tiffDouble)</TD></TR>
+          <TR><TD class="indexkey">tiffFloat</TD><TD class="indexvalue">%ValueType &lt; float &gt;</TD></TR>
+          <TR><TD class="indexkey">tiffDouble</TD><TD class="indexvalue">%ValueType &lt; double &gt;</TD></TR>
           <TR><TD class="indexkey">tiffIfd</TD><TD class="indexvalue">%ValueType &lt; uint32_t &gt;</TD></TR>
           <TR><TD class="indexkey">date</TD><TD class="indexvalue">%DateValue</TD></TR>
           <TR><TD class="indexkey">time</TD><TD class="indexvalue">%TimeValue</TD></TR>
@@ -1209,6 +1210,10 @@ namespace Exiv2 {
     template<> inline TypeId getType<int32_t>() { return signedLong; }
     //! Specialization for a signed rational
     template<> inline TypeId getType<Rational>() { return signedRational; }
+    //! Specialization for a float
+    template<> inline TypeId getType<float>() { return tiffFloat; }
+    //! Specialization for a double
+    template<> inline TypeId getType<double>() { return tiffDouble; }
 
     // No default implementation: let the compiler/linker complain
     // template<typename T> inline TypeId getType() { return invalid; }
@@ -1225,12 +1230,15 @@ namespace Exiv2 {
 
         //! @name Creators
         //@{
-        //! Default constructor.
+        //! Default Constructor.
         ValueType();
-        //! Constructor
-        ValueType(const byte* buf, long len, ByteOrder byteOrder);
-        //! Constructor (Note: Argument \em byteOrder is ignored and deprecated)
-        explicit ValueType(const T& val, ByteOrder byteOrder =littleEndian);
+        //! Constructor.
+        // The default c'tor and this one can be combined, but that causes MSVC 7.1 to fall on its nose
+        explicit ValueType(TypeId typeId);
+        //! Constructor.
+        ValueType(const byte* buf, long len, ByteOrder byteOrder, TypeId typeId =getType<T>());
+        //! Constructor.
+        explicit ValueType(const T& val, TypeId typeId =getType<T>());
         //! Copy constructor
         ValueType(const ValueType<T>& rhs);
         //! Virtual destructor.
@@ -1321,6 +1329,10 @@ namespace Exiv2 {
     typedef ValueType<int32_t> LongValue;
     //! Signed rational value type
     typedef ValueType<Rational> RationalValue;
+    //! Float value type
+    typedef ValueType<float> FloatValue;
+    //! Double value type
+    typedef ValueType<double> DoubleValue;
 
 // *****************************************************************************
 // free functions, template and inline definitions
@@ -1373,6 +1385,18 @@ namespace Exiv2 {
     {
         return getRational(buf, byteOrder);
     }
+    // Specialization for a 4 byte float value.
+    template<>
+    inline float getValue(const byte* buf, ByteOrder byteOrder)
+    {
+        return getFloat(buf, byteOrder);
+    }
+    // Specialization for a 8 byte double value.
+    template<>
+    inline double getValue(const byte* buf, ByteOrder byteOrder)
+    {
+        return getDouble(buf, byteOrder);
+    }
 
     /*!
       @brief Convert a value of type T to data, write the data to the data buffer.
@@ -1441,6 +1465,24 @@ namespace Exiv2 {
     {
         return r2Data(buf, t, byteOrder);
     }
+    /*!
+      @brief Specialization to write a float to the data buffer.
+             Return the number of bytes written.
+     */
+    template<>
+    inline long toData(byte* buf, float t, ByteOrder byteOrder)
+    {
+        return f2Data(buf, t, byteOrder);
+    }
+    /*!
+      @brief Specialization to write a double to the data buffer.
+             Return the number of bytes written.
+     */
+    template<>
+    inline long toData(byte* buf, double t, ByteOrder byteOrder)
+    {
+        return d2Data(buf, t, byteOrder);
+    }
 
     template<typename T>
     ValueType<T>::ValueType()
@@ -1449,15 +1491,21 @@ namespace Exiv2 {
     }
 
     template<typename T>
-    ValueType<T>::ValueType(const byte* buf, long len, ByteOrder byteOrder)
-        : Value(getType<T>()), pDataArea_(0), sizeDataArea_(0)
+    ValueType<T>::ValueType(TypeId typeId)
+        : Value(typeId), pDataArea_(0), sizeDataArea_(0)
+    {
+    }
+
+    template<typename T>
+    ValueType<T>::ValueType(const byte* buf, long len, ByteOrder byteOrder, TypeId typeId)
+        : Value(typeId), pDataArea_(0), sizeDataArea_(0)
     {
         read(buf, len, byteOrder);
     }
 
     template<typename T>
-    ValueType<T>::ValueType(const T& val, ByteOrder /*byteOrder*/)
-        : Value(getType<T>()), pDataArea_(0), sizeDataArea_(0)
+    ValueType<T>::ValueType(const T& val, TypeId typeId)
+        : Value(typeId), pDataArea_(0), sizeDataArea_(0)
     {
         value_.push_back(val);
     }
@@ -1560,7 +1608,7 @@ namespace Exiv2 {
         typename ValueList::const_iterator end = value_.end();
         typename ValueList::const_iterator i = value_.begin();
         while (i != end) {
-            os << *i;
+            os << std::setprecision(15) << *i;
             if (++i != end) os << " ";
         }
         return os;
@@ -1578,7 +1626,7 @@ namespace Exiv2 {
     long ValueType<T>::toLong(long n) const
     {
         ok_ = true;
-        return value_[n];
+        return static_cast<long>(value_[n]);
     }
     // Specialization for rational
     template<>
@@ -1640,6 +1688,22 @@ namespace Exiv2 {
         ok_ = true;
         return Rational(value_[n].first, value_[n].second);
     }
+    // Specialization for float.
+    template<>
+    inline Rational ValueType<float>::toRational(long n) const
+    {
+        ok_ = true;
+        // Warning: This is a very simple conversion, see floatToRationalCast()
+        return floatToRationalCast(value_[n]);
+    }
+    // Specialization for double.
+    template<>
+    inline Rational ValueType<double>::toRational(long n) const
+    {
+        ok_ = true;
+        // Warning: This is a very simple conversion, see floatToRationalCast()
+        return floatToRationalCast(static_cast<float>(value_[n]));
+    }
 
     template<typename T>
     long ValueType<T>::sizeDataArea() const
